#ifndef GHS_CONTROL_STRUCTS_H__
#define GHS_CONTROL_STRUCTS_H__

#include "stdbool.h"
#include "boards.h"

#define OPCODE_LENGTH  1                        /**< Length of opcode inside PO Measurement packet. */
#define HANDLE_LENGTH  2                        /**< Length of handle inside PO Measurement packet. */
#define MAX_CHAR_LEN   (BLE_L2CAP_MTU_MIN - OPCODE_LENGTH - HANDLE_LENGTH)  /**< Maximum size of a transmitted Measurement. */

#define BTLE_DEVICE_INFORMATION_SERVICE         0x180A
#define BTLE_DIS_MANUFACTURER_DEVICE_NAME_CHAR  0x2A29
#define BTLE_DIS_MODEL_NUMBER_CHAR              0x2A24
#define BTLE_DIS_SERIAL_NUMBER_CHAR             0x2A25
#define BTLE_DIS_FIRMWARE_REVISION_CHAR         0x2A26
#define BTLE_DIS_HARDWARE_REVISION_CHAR         0x2A27
#define BTLE_DIS_SOFTWARE_REVISION_CHAR         0x2A28
#define BTLE_DIS_REG_CERT_DATA_LIST_CHAR        0x2A2A
#define BTLE_DIS_SYSTEM_ID_CHAR                 0x2A23

#define BTLE_CYCLE_POWER_SERVICE                0x1818
#define BTLE_CYCLE_POWER_MSMT_CHAR              0x2A63
#define BTLE_CYCLE_POWER_FEATURE_CHAR           0x2A65
#define BTLE_SENSOR_LOCATION_CHAR               0x2A5D
#define BTLE_SCCD_DESCRIPTOR                    0x2903

#define BTLE_HEART_RATE_SERVICE                 0x180D
#define BTLE_HEART_RATE_MSMT_CHAR               0x2A37

// Locations
#define OTHER 0
#define TOP_OF_SHOE 1
#define IN_SHOE 2
#define HIP 3
#define FRONT_WHEEL 4
#define LEFT_CRANK 5
#define RIGHT_CRANK 6
#define LEFT_PEDAL 7
#define RIGHT_PEDAL 8
#define FRONT_HUB 9
#define REAR_DROPOUT 10
#define CHAINSTAY 11
#define REAR_WHEEL 12
#define REAR_HUB 13
#define CHEST 14
#define SPIDER 15
#define CHAIN_RING 16

// Cycling power feature
#define FEATURE_PEDAL_POWER_BALANCE_SUPPORTED   0x01    //1
#define FEATURE_ACCUMULATED_TORQUE_SUPPORTED    0x02    //2
#define FEATURE_WHEEL_REVOLUTION_DATA_SUPPORTED 0x04    //4
#define FEATURE_CRANK_REVOLUTION_DATA_SUPPORTED 0x08    //8
#define FEATURE_EXTREME_MAGNITUDES_SUPPORTED    0x10    //16
#define FEATURE_EXTREME_ANGLES_SUPPORTED        0x20    //32
#define FEATURE_TOP_AND_BOTTOM_DEAD_SPOT_ANGLES_SUPPORTED 0x40   //64
#define FEATURE_ACCUMULATED_ENERGY_SUPPORTED              0x80   //128
#define FEATURE_OFFSET_COMPENSATION_INDICATOR_SUPPORTED   0x100  //256
#define FEATURE_OFFSET_COMPENSATION_SUPPORTED             0x200  //512
#define FEATURE_CYCLING_POWER_MEASUREMENT_CHAR_MASKING_SUPPORTED 0x400      //1024
#define FEATURE_MULTIPLE_SENSOR_LOCATIONS_SUPPORTED              0x800      //2048
#define FEATURE_CRANK_LENGTH_ADJUSTMENT_SUPPORTED                0x1000     //4096
#define FEATURE_CHAIN_LENGTH_ADJUSTMENT_SUPPORTED                0x2000     //8192
#define FEATURE_CHAIN_WEIGHT_ADJUSTMENT_SUPPORTED                0x4000     //16384
#define FEATURE_SPAN_LENGTH_ADJUSTMENT_SUPPORTED                 0x8000     //32768
#define FEATURE_SENSOR_MEASUREMENT_CONTEXT_TORQUE_BASED          0x10000    //65536 -- Otherwise force based
#define FEATURE_INSTANTANEOUS_MEASUREMENT_DIRECTION_SUPPORTED    0x20000    //131072
#define FEATURE_FACTORY_CALIBRATION_DATE_SUPPORTED               0x40000    //262144
#define FEATURE_ENHANCED_OFFSET_COMPENSATION_PROCEDURE_SUPPORTED 0x80000    //524288
#define FEATURE_NOT_FOR_USE_IN_DISTRIBUTED_SYSTEM                0x100000   //1048576 need (feature & 0x300000) == 0x100000
#define FEATURE_MAY_BE_USED_IN_DISTRIBUTED_SYSTEM                0x200000   //2097152 need (feature & 0x300000) == 0x200000

// Cycling Power measurement flags
#define CP_MSMT_FLAGS_HAS_PEDAL_POWER_BALANCE 1
#define CP_MSMT_FLAGS_PEDAL_POWER_BALANCE_REF_LEFT 2        // Otherwise unknown
#define CP_MSMT_FLAGS_HAS_ACCUMULATED_TORQUE 4
#define CP_MSMT_FLAGS_ACCUMULATED_TORQUE_SOURCE_CRANK 8     // Otherwise wheel
#define CP_MSMT_FLAGS_HAS_WHEEL_REVS 0x10
#define CP_MSMT_FLAGS_HAS_CRANK_REVS 0x20
#define CP_MSMT_FLAGS_HAS_EXTREME_FORCE_MAGNITUDES 0x40
#define CP_MSMT_FLAGS_HAS_EXTREME_TORQUE_MAGNITUDES 0x80
#define CP_MSMT_FLAGS_HAS_EXTREME_ANGLES 0x100
#define CP_MSMT_FLAGS_HAS_TOP_DEAD_SPOT_ANGLE 0x200
#define CP_MSMT_FLAGS_HAS_BOTTOM_DEAD_SPOT_ANGLE 0x400
#define CP_MSMT_FLAGS_HAS_ACCUMULATED_ENERGY 0x800
#define CP_MSMT_FLAGS_HAS_OFFSET_COMPENSATION 0x1000

// Cycling Speed and Cadence Feature
#define CSC_FEATURE_WHEEL_REVS_SUPPORTED   0x01    //1
#define CSC_FEATURE_CRANK_REVS_SUPPORTED   0x02
#define CSC_FEATURE_MULT_SENSOR_LOCATIONS_SUPPORTED   0x04

// Cycling Speed and Cadence measurement flags
#define CSC_MSMT_FLAGS_HAS_WHEEL_REVS 1
#define CSC_MSMT_FLAGS_HAS_CRANK_REVS 1

// Cycling Speed and Cadence measurement
// flags 1 byte
// cumulative wheel revolutions 4 bytes  (present if flags bit 0 set)
// last wheel event time 2 bytes (present if flags bit 0 set) in units of 1024ths of a sec
// cumulative crank revolutions 2 bytes  (present if flags bit 1 set)
// last crank event time 2 bytes (present if flags bit 2 set) in units of 1024ths of a sec

#define HEART_RATE_MSMT_FLAGS_TWO_BYTES 0x01

// LED_0 is the other LED which is green-yellow
// LED_1 is red
// LED_2 is green
// LED_3 is blue

#define ADVERTISING_LED                 BSP_BOARD_LED_2                         /**< Is on when device is advertising. */ 
#define RED_LED                         BSP_BOARD_LED_1
#define BLUE_LED                        BSP_BOARD_LED_3                         /**< Is on when device has connected.*/
#define DISCONNECTED_LED                BSP_BOARD_LED_0

#define USE_DK 0        // Set NRF_LOG_ENABLED to 0 when DK is 0. The idea is either DK or nRF52840 dongle
                        // Board in preprocessor needs to be changed from BOARD_PCA10056 (DK) to BOARD_PCA10059 (dongle)


typedef struct
{
    unsigned char systemId[8];              // 8 bytes little endian for the EUI-64 system Id
    unsigned short regCertDataListLength;   // Length of the RegCertData list
    unsigned char* regCertDataList;         // RegCertData list in Mder Format (as an IEEE ASN.1 Struct)
    char* serialNumber;        // String
    char* firmwareRevision;    // String
    char* hardwareRevision;    // String
    char* softwareRevision;    // String
    char* manufacturerName;    // String
    char* modelNumber;         // String
    bool trapRead;                      // If true, read attempts will be evented
    bool useDefaultRegCertDataList;     // If true, a fixed reg cert data list for Continua 6.0 and unregulated will be created

} s_DisCharValues;

#define COUNT_MASK 0x00

typedef struct
{
    unsigned short power;       // units are watts
    unsigned short rpm;         // units are rpm * 10
    unsigned short crankTime;   // units are 1024ths of a second
    unsigned short hr;          // units are beats per minute
} s_PowerMsmt;

#endif
